module Logvisible
  module LogMethods
    LOG_LEVELS = [
      :debug,
      :info,
      :warn,
      :error,
      :fatal
    ]

    STYLES = {
      bold:      "\e[1m",
      underline: "\e[4m",
      inverse:   "\e[7m"
    }

    COLORS = {
      black:   "\e[30m",
      red:     "\e[31m",
      green:   "\e[32m",
      yellow:  "\e[33m",
      blue:    "\e[34m",
      magenta: "\e[35m",
      cyan:    "\e[36m",
      white:   "\e[37m"
    }

    RESET = "\e[0m"

    def self.included(base)
      base.extend self
    end

    def logvisible(string, opts={})
      opts  = { sep: "\n", level: :debug }.merge opts
      level = _validate_log_level opts[:level]
      sep   = opts[:sep].to_s * 5
      msg   = sep + string.to_s + sep
      _decorate!(msg, opts[:text]) if opts[:text]
      logger.send level, msg
    end

    # Decorates messages passed to Rails' TaggedLogger
    def logvisible_location(msg, opts={})
      return logvisible(msg, opts) unless logger.respond_to? :tagged
      opts = { location: find_logvisible_location, additional_tag: '', level: :debug }.merge opts
      tag  = opts[:location].to_s + opts[:additional_tag].to_s
      _decorate! tag, opts[:text] if opts[:text]
      _decorate! msg, opts[:text] if opts[:text]
      logger.tagged(tag) do
        logvisible msg, { sep: '', level: opts[:level] }
      end
    end

    private

    def logger
      Logvisible.logger
    end

    def find_logvisible_location
      match    = caller[1].match(/`(\w+)'/)
      method   = match ? match.captures.first : ''
      location = Class == self.class ? self.name : "#{self.class.name}##{method}"
    end

    def _decorate!(string, opts={})
      style = _get_styles opts[:style]
      color = _get_color opts[:color]
      string.insert(0, color)
      string.insert(0, style)
      string.insert(-1, _reset!) if string.start_with? "\e["
    end

    def _reset!
      RESET
    end

    def _get_styles(style)
      if style.is_a? Array
        style.inject('') { |str, s| str += STYLES[s].to_s }
      else
        STYLES[style].to_s
      end
    end

    def _get_color(color)
      COLORS[color].to_s
    end

    def _validate_log_level(level)
      level = :debug unless LOG_LEVELS.include? level
      level
    end
  end
end
